# Benchmarks

## Sample apps

All apps employ two global middlewares with `req` mutations, an empty `GET` route handler for `favicon.ico` and a GET handler for the `/users/:id`, returning a `User: {id}` string response.

## Benchmarking command

Results are taken after one run. The command used for results is the following:

```sh
node run.mjs
```

`run.mjs` is a script that launches a child process with the app and runs benchmarks against it, recursively, going to the next app in the list.

## Information

### Hardware

Xiaomi Laptop with 16GB RAM and Intel Core i7-8550U processor.

### Software

- **OS**: Manjaro Linux
- **Kernel version**: 5.7.19-2-MANJARO
- **Node.js**: 14.9.0

## Results

> **Note**: benchmarks aren't completely accurate and are different on every run and on every machine. You need to compare proportions instead of absolute values.

The table takes average results.

| framework                  | req/s | transfer/sec | latency  |
| -------------------------- | ----- | ------------ | -------- |
| @tinyhttp/app (esm)        | 17709 | 2.32 MB      | 5.3 ms   |
| @tinyhttp/app (cjs)        | 14264 | 1.87 MB      | 6.48 ms  |
| express@4.17.1             | 7420  | 1.14 MB      | 12.86 ms |
| polka@0.5.2                | 21804 | 2.86 MB      | 4.25 ms  |
| koa@2.13 + @koa/router@9.4 | 14219 | 2.45 MB      | 6.45 ms  |

**Conclusion**: tinyhttp is ~2.3x faster than Express and ~1.2x slower than Polka.

## Detailed results

- tinyhttp (esm)

```

┌─────────┬──────┬──────┬───────┬───────┬────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%   │ Avg    │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼───────┼────────┼─────────┼───────┤
│ Latency │ 5 ms │ 5 ms │ 7 ms  │ 12 ms │ 5.3 ms │ 2.09 ms │ 90 ms │
└─────────┴──────┴──────┴───────┴───────┴────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬─────────┬─────────┬─────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%     │ 97.5%   │ Avg     │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼─────────┼─────────┼─────────┼─────────┼─────────┤
│ Req/Sec   │ 8495    │ 8495    │ 18383   │ 19007   │ 17709.6 │ 2506.71 │ 8489    │
├───────────┼─────────┼─────────┼─────────┼─────────┼─────────┼─────────┼─────────┤
│ Bytes/Sec │ 1.11 MB │ 1.11 MB │ 2.41 MB │ 2.49 MB │ 2.32 MB │ 329 kB  │ 1.11 MB │
└───────────┴─────────┴─────────┴─────────┴─────────┴─────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

266k requests in 15.08s, 34.8 MB read
```

- tinyhttp (cjs)

```
┌─────────┬──────┬──────┬───────┬───────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%   │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼───────┼─────────┼─────────┼───────┤
│ Latency │ 5 ms │ 6 ms │ 12 ms │ 17 ms │ 6.48 ms │ 2.55 ms │ 52 ms │
└─────────┴──────┴──────┴───────┴───────┴─────────┴─────────┴───────┘
┌───────────┬────────┬────────┬─────────┬────────┬─────────┬────────┬────────┐
│ Stat      │ 1%     │ 2.5%   │ 50%     │ 97.5%  │ Avg     │ Stdev  │ Min    │
├───────────┼────────┼────────┼─────────┼────────┼─────────┼────────┼────────┤
│ Req/Sec   │ 8423   │ 8423   │ 14767   │ 18351  │ 14264.8 │ 2906.1 │ 8418   │
├───────────┼────────┼────────┼─────────┼────────┼─────────┼────────┼────────┤
│ Bytes/Sec │ 1.1 MB │ 1.1 MB │ 1.94 MB │ 2.4 MB │ 1.87 MB │ 381 kB │ 1.1 MB │
└───────────┴────────┴────────┴─────────┴────────┴─────────┴────────┴────────┘

Req/Bytes counts sampled once per second.

214k requests in 15.04s, 28 MB read
300 errors (0 timeouts)
```

- express@4.17.1

```
┌─────────┬───────┬───────┬───────┬───────┬──────────┬─────────┬───────┐
│ Stat    │ 2.5%  │ 50%   │ 97.5% │ 99%   │ Avg      │ Stdev   │ Max   │
├─────────┼───────┼───────┼───────┼───────┼──────────┼─────────┼───────┤
│ Latency │ 11 ms │ 12 ms │ 22 ms │ 26 ms │ 12.86 ms │ 3.22 ms │ 56 ms │
└─────────┴───────┴───────┴───────┴───────┴──────────┴─────────┴───────┘
┌───────────┬────────┬────────┬────────┬─────────┬─────────┬────────┬────────┐
│ Stat      │ 1%     │ 2.5%   │ 50%    │ 97.5%   │ Avg     │ Stdev  │ Min    │
├───────────┼────────┼────────┼────────┼─────────┼─────────┼────────┼────────┤
│ Req/Sec   │ 3391   │ 3391   │ 7803   │ 8199    │ 7420.34 │ 1159.5 │ 3390   │
├───────────┼────────┼────────┼────────┼─────────┼─────────┼────────┼────────┤
│ Bytes/Sec │ 522 kB │ 522 kB │ 1.2 MB │ 1.26 MB │ 1.14 MB │ 179 kB │ 522 kB │
└───────────┴────────┴────────┴────────┴─────────┴─────────┴────────┴────────┘

Req/Bytes counts sampled once per second.

111k requests in 15.04s, 17.1 MB read
500 errors (0 timeouts)
```

- polka@0.5.2

```
┌─────────┬──────┬──────┬───────┬──────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%  │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼──────┼─────────┼─────────┼───────┤
│ Latency │ 3 ms │ 4 ms │ 7 ms  │ 9 ms │ 4.25 ms │ 1.33 ms │ 44 ms │
└─────────┴──────┴──────┴───────┴──────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬───────┬─────────┬──────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%   │ 97.5%   │ Avg      │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼───────┼─────────┼──────────┼─────────┼─────────┤
│ Req/Sec   │ 13199   │ 13199   │ 22927 │ 23583   │ 21804.54 │ 2711.17 │ 13193   │
├───────────┼─────────┼─────────┼───────┼─────────┼──────────┼─────────┼─────────┤
│ Bytes/Sec │ 1.73 MB │ 1.73 MB │ 3 MB  │ 3.09 MB │ 2.86 MB  │ 355 kB  │ 1.73 MB │
└───────────┴─────────┴─────────┴───────┴─────────┴──────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

327k requests in 15.03s, 42.8 MB read
200 errors (0 timeouts)
```

- koa@2.13 + @koa/router@9.4

```
┌─────────┬──────┬──────┬───────┬───────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%   │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼───────┼─────────┼─────────┼───────┤
│ Latency │ 5 ms │ 6 ms │ 13 ms │ 18 ms │ 6.45 ms │ 2.63 ms │ 62 ms │
└─────────┴──────┴──────┴───────┴───────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬─────────┬────────┬─────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%     │ 97.5%  │ Avg     │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼─────────┼────────┼─────────┼─────────┼─────────┤
│ Req/Sec   │ 6147    │ 6147    │ 15831   │ 16895  │ 14219.6 │ 3110.84 │ 6144    │
├───────────┼─────────┼─────────┼─────────┼────────┼─────────┼─────────┼─────────┤
│ Bytes/Sec │ 1.06 MB │ 1.06 MB │ 2.72 MB │ 2.9 MB │ 2.45 MB │ 535 kB  │ 1.06 MB │
└───────────┴─────────┴─────────┴─────────┴────────┴─────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

213k requests in 15.03s, 36.7 MB read
600 errors (0 timeouts)
```
